#include <sys/cdefs.h>
#include <asm/types.h>
#include <linux/skbuff.h>
#include <linux/in.h>

#include "radij.h"
#include "ipsec_encap.h"
#include "ipsec_netlink.h"
#include "ipsec_xform.h"
#include "ipsec_ah.h"
#include "ipsec_esp.h"

typedef __u32 in_addr_t;

__u32 arc4random(void);
